/*
 * Copyright 2022-2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package xyz.zhouxy.plusone.commons.exception;

import xyz.zhouxy.plusone.commons.base.IWithCode;

import javax.annotation.Nonnull;
import java.util.Objects;

/**
 * 带错误码的异常。
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public abstract class BaseException
        extends Exception
        implements IWithCode<String> {

    private static final long serialVersionUID = -2546365325001947203L;

    @Nonnull
    private final String code;

    protected BaseException(String code, String msg) {
        super(msg);
        this.code = Objects.requireNonNull(code);
    }

    protected BaseException(String code, Throwable cause) {
        super(cause);
        this.code = Objects.requireNonNull(code);
    }

    protected BaseException(String code, String msg, Throwable cause) {
        super(msg, cause);
        this.code = Objects.requireNonNull(code);
    }

    @Nonnull
    @Override
    public final String getCode() {
        return this.code;
    }
}
